# -*- coding: utf-8 -*-
import pymysql
from logzero import logger
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from XX.Date.DatetimeHelper import get_now_time

# TODO:优化一下

global_conn = None


def get_session(host="localhost", user="root", password="123456", port="3306", charset="utf-8", db="test", *arg, **kw):
    global session
    if session:
        return session
    else:
        driver = kw.get("driver", "pymysql")
        mysql_url = 'mysql+' + driver + '://' + user + ":" + password + "@" + host + ":" + str(port) + "/" + str(
            db) + "?charset=" + charset.replace("-", "") + "&autocommit=true"
        engine = create_engine(mysql_url, encoding="utf8", *arg, **kw)
        session = sessionmaker(bind=engine)()
        return session


def connect(host="localhost", user="root", password="123456", port=3306, charset="utf8", db="test",
            mysql_timeout=5000, use_global=False):
    global global_conn
    if global_conn:
        logger.info("使用全局连接")
        return global_conn

    try_times = 3
    conn = None
    for i in range(try_times):
        try:
            conn = pymysql.connect(host=host, user=user, password=password, db=db, port=port, charset=charset,
                                   connect_timeout=mysql_timeout, autocommit=1)
            break
        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.info("data connection error,%s" % e)
    if use_global:
        global_conn = conn
        logger.info("首次连接")
    return conn


def close(conn):
    global global_conn
    if global_conn:
        return False
    else:
        pass
    try:
        conn.close()
    except Exception as e:
        logger.info("data shutdown error,%s" % e)

    return True


def insert(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor(pymysql.cursors.DictCursor)
        cur.execute(sql)
        id_ = int(conn.insert_id())
        conn.commit()
    except Exception as e:
        logger.info("Mysql Insert Error: %s, %s" % (e, sql))
        logger.info("data insert error")
        conn.rollback()
        id_ = -1
    return id_


def lastrowid(conn):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor()
        id_ = int(cur.lastrowid)
    except Exception as e:
        logger.info("data last_id error,%s" % e)
        id_ = -1
    return id_


def update(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor(pymysql.cursors.DictCursor)
        result = cur.execute(sql)
        conn.commit()
    except Exception as e:
        logger.info(f" ========== Mysql Update Error: {e}  sql :  {sql}")
        open("mysql-err.log", "a", encoding="utf-8").write(get_now_time() + sql + "\n")
        conn.ping()
        conn.rollback()
        result = False
    return result


def fetch(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor(pymysql.cursors.DictCursor)
        rs = cur.execute(sql)
        rows = cur.fetchmany(rs)
        conn.commit()
    except Exception as e:
        logger.info(f"Mysql Fetch Error: {e}\n{sql}")
        conn.rollback()
        conn.ping()
        rows = []
    return rows


def make_insert_sql(table: str, data: dict, updates: dict, replace=1) -> str:
    columns_sql = ""
    values_sql = ""
    updates_sql = ""
    for key in data.keys():
        if data[key]:
            data[key] = pymysql.converters.escape_string(str(data[key]))
        columns_sql = columns_sql + "`" + key + "`,\t"
        if data[key] is not None:
            values_sql = values_sql + "'" + str(data[key]) + "',\t"
        else:
            values_sql = values_sql + "null,\t"

        if replace:
            # 覆盖
            if key in updates:
                if data[key] is not None:
                    updates_sql = updates_sql + "`" + key + "` = '" + str(data[key]) + "',\t"
                else:
                    updates_sql = updates_sql + "`" + key + "` = null,\t"
        else:
            # 补充
            if key in updates:
                if data[key]:
                    updates_sql = updates_sql + "`" + key \
                                  + f"` =  CASE WHEN {key}  IS NOT NULL THEN {key} ELSE '" + str(
                        data[key]) + "'  END,\t"

    columns_sql = columns_sql[:-2]
    values_sql = values_sql[:-2]
    if len(updates_sql) > 2:
        updates_sql = updates_sql[:-2]
    return f"INSERT INTO `{table}`( {columns_sql} )VALUES({values_sql}) ON DUPLICATE KEY UPDATE {updates_sql} ;\t"


def make_update_sql(table: str, data: dict, replace=1) -> str:
    sql = f"UPDATE `{table}` SET "
    update_ = ''
    for k, v in data.items():
        if replace:
            if v is not None:
                update_ += f",{k.strip()}='{pymysql.converters.escape_string(str(v))}' \n "
            else:
                update_ += f",{k.strip()}=null \n"
        else:
            if v is not None:
                update_ += f",{k.strip()}=CASE WHEN {k} IS NOT NULL THEN {k} ELSE '{pymysql.converters.escape_string(str(v))}' END \n"
            else:
                update_ += f",{k.strip()}=CASE WHEN {k} IS NOT NULL THEN {k} ELSE null END \n"
    update_ = update_.lstrip(",")
    sql += update_ + f" WHERE id = '{data['id']}' "
    return sql



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/16 16:53
# @Author   : Peter
# @Des       : 
# @File        : SqlAlchemyHelper
# @Software: PyCharm
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


class SqlAlchemyHelper():

    @staticmethod
    def get_session(host="localhost", user="root", pwd="root", port=3306, charset="utf-8", db="test", *arg, **kw):
        driver = kw.get("driver", "pymysql")
        mysql_url = 'mysql+' + driver + '://' + user + ":" + pwd + "@" + host + ":" + str(port) + "/" + str(db) + "?charset=" + charset.replace("-", "")
        engine = create_engine(mysql_url, encoding=charset, pool_size=10, max_overflow=20, *arg, **kw)
        return sessionmaker(bind=engine)()

    @staticmethod
    def get_session_by_cfg(cfg):
        return SqlAlchemyHelper.get_session(**cfg)


if __name__ == "__main__":
    import XX.Model.Struct.MysqlConn as Mcfg

    sa = SqlAlchemyHelper()
    mysql_cfg = Mcfg.ubuntu_cfg
    mysql_cfg["db"] = "weibo"
    print(mysql_cfg)
    session = sa.get_session_by_cfg(mysql_cfg)
    print(session)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/1/17 11:09
# @File           : Mysql2Redis
# @Des           :
# @Email        : billsteve@126.com
import pymysql
import requests
from dbutils.pooled_db import PooledDB

from XX.List.ListHelper import ListHelper


class MysqlPoolHelper:
    __pool = None
    db = None
    host = None

    def __init__(self, host="127.0.0.1", user="root", pwd="root", db="test", port=3306, charset="utf8", kv=1, **kw):
        self._conn = MysqlPoolHelper.get_conn(host, user, pwd, db, port, charset.replace("-", ""))
        if kv:
            self.cur = self._conn.cursor(cursor=pymysql.cursors.DictCursor)
        else:
            self.cur = self._conn.cursor()

    @staticmethod
    def get_conn(host, user, pwd, db, port=3306, charset="utf8"):
        if MysqlPoolHelper.__pool is None or MysqlPoolHelper.db != db or MysqlPoolHelper.host != host:
            MysqlPoolHelper.db = db
            MysqlPoolHelper.__pool = PooledDB(creator=pymysql, host=host, port=port, user=user, passwd=pwd, db=db, use_unicode=False, charset=charset)
            print(host, port, user, pwd, db, charset)
        return MysqlPoolHelper.__pool.connection() if MysqlPoolHelper.__pool else None

    def get_lists(self, sql, values=None):
        self.cur.execute(sql, values)
        res = ListHelper.decode_v(self.cur.fetchall())
        return res

    def exec_query(self, sql, values=None):
        res = self.cur.execute(sql, values)
        self._conn.commit()
        return res

    def exec_query_no_commit(self, sql, values=None):
        return self.cur.execute(sql, values)

    def commit(self):
        self._conn.commit()

    def rollback(self):
        self._conn.rollback()

    def get_last_insert_id(self):
        return self.cur.lastrowid

    def closeConn(self):
        self._conn.close()

    def dispose(self, method="commit"):
        if method == "commit":
            self._conn.commit()
        else:
            self._conn.rollback()
        self.cur.close()
        self._conn.close()

    def check_info(self, url, data):
        requests.post(url, data=data)


if __name__ == "__main__":
    DBHOST1 = "localhost"
    DBPORT = 3306
    DBUSER = "root"
    DBPWD = "DRsXT5ZJ6Oi55LPQ"
    DBNAME = "test"
    DBCHAR = "utf8"

    conn = MysqlPoolHelper(DBHOST1, DBUSER, DBPWD, DBNAME)
    print(conn)



# -*- coding:utf-8 -*-
# coding: utf-8
from sqlalchemy import Column, Integer, String, Float
from sqlalchemy.ext.declarative import declarative_base
from XX.Model.SqlAlchemy.BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class WxCount(Base, BaseModel):
    __tablename__ = 'wx_count'

    id = Column(Integer, primary_key=True)
    ReportMonth = Column(String(255))
    UserId = Column(String(255))
    PostTimes = Column(Integer)
    ArticleCount = Column(Integer)
    ReadNumSum = Column(String(255))
    ReadNumMax = Column(Integer)
    ReadNumAvg = Column(Integer)
    LikeNumSum = Column(String(255))
    TopReadNumAvg = Column(Integer)
    BangIndex = Column(Float)
    UserName = Column(String(255))
    Alias = Column(String(255))
    Description = Column(String(255))
    HeadImgUrl = Column(String(255))
    WechatId = Column(String(255))

    def __init__(self, *arg, **kw):
        self.Alias = kw.get("Alias", None)
        self.ArticleCount = kw.get("ArticleCount", None)
        self.BangIndex = kw.get("BangIndex", None)
        self.Description = kw.get("Description", None)
        self.HeadImgUrl = kw.get("HeadImgUrl", None)
        self.LikeNumSum = kw.get("LikeNumSum", None)
        self.PostTimes = kw.get("PostTimes", None)
        self.ReadNumAvg = kw.get("ReadNumAvg", None)
        self.ReadNumMax = kw.get("ReadNumMax", None)
        self.ReadNumSum = kw.get("ReadNumSum", None)
        self.ReportMonth = kw.get("ReportMonth", None)
        self.TopReadNumAvg = kw.get("TopReadNumAvg", None)
        self.UserId = kw.get("UserId", None)
        self.UserName = kw.get("UserName", None)
        self.WechatId = kw.get("WechatId", None)
        self.id = kw.get("id", None)
        self.metadata = kw.get("metadata", None)

    @staticmethod
    def getInfoByUserId(UserId, session):
        infos = session.query(WxCount).filter(WxCount.UserId == UserId).all()
        return infos[0].id if infos else None

    @staticmethod
    def getInfoByAlias(Alias, session):
        infos = session.query(WxCount).filter(WxCount.Alias == Alias).all()
        return infos[0].id if infos else None

    @staticmethod
    def getByFromId(from_id, session):
        return session.query(WxCount).filter(WxCount.id > from_id).limit(100).all()


if __name__ == '__main__':
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(WxCount):
        if not (str(k).startswith("_") or isinstance(getattr(WxCount, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/16 16:37
# @Author   : Peter
# @Des       : 
# @File        : PostPic
# @Software: PyCharm
from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from XX.Model.SqlAlchemy.BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class PostPicModel(Base, BaseModel):
    __tablename__ = 'bbs_post_pic'

    id = Column(Integer, primary_key=True)
    post_id = Column(Integer)
    pic_url = Column(String(100))
    location = Column(String(100))
    head_pic = Column(Integer)
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.id = kw.get("id", None)
        self.post_id = kw.get("post_id", None)
        self.pic_url = kw.get("pic_url", None)
        self.location = kw.get("location", None)
        self.head_pic = kw.get("head_pic", None)
        self.is_del = kw.get("is_del", None)
        self.create_ts = kw.get("create_ts", None)
        self.update_ts = kw.get("update_ts", None)

    @staticmethod
    def getByPostIdPicUrl(post_id, pic_url, session):
        return session.query(PostPicModel).filter(PostPicModel.post_id == post_id, PostPicModel.pic_url == pic_url).all()



# -*- coding: utf-8 -*-
import requests

from XX.HTML.UrlHelper import UrlHelper as Uh


def google_translate(txt):
    q = Uh.url_encode(txt)
    url = f"https://translate.google.cn/translate_a/single?client=webapp&sl=auto&tl=zh-CN&hl=zh-CN&dt=at&dt=bd&dt=ex&dt=ld&dt=md&dt=qca&dt=rw&dt=rm&dt=sos&dt=ss&dt=t&ssel=0&tsel=3&kc=0&tk=550486.961169&q={q}"
    r = requests.get(url)
    return r.json()[0][0][0]



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2019/2/23 18:15
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Headless.py
# @Des         : 
# @Software : PyCharm
from selenium import webdriver


def get(url):
    options = webdriver.FirefoxOptions()
    options.set_headless()
    options.add_argument('-headless')
    options.add_argument('--disable-gpu')
    driver = webdriver.Firefox(firefox_options=options)
    driver.get(url)
    driver.close()
    html = driver.page_source
    return html


if __name__ == '__main__':
    print(get("http://httpbin.org/get"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/1/22 10:16
# @File           : RecrawlNot200
# @Des           : 
# @Email        : billsteve@126.com
import time

import XX.Model.Struct.RedisConn as RC
import XX.BuiltinFunctions as BF
from XX.DB.RedisHelper import *
from logzero import logger


# @deprecated
def re_add_not200(redis_cfg=RC.ali2_cfg(db=0), ts=10):
    conn_redis = RedisHelper.get_redis_connect_by_cfg(redis_cfg)
    while 1:
        keys = conn_redis.keys("*not200*")
        if not keys:
            BF.print_from_head("No More not 200 Spider in " + str(redis_cfg["host"]), ts=ts)
            continue
        for key in keys:
            url = conn_redis.spop(key)
            if url:
                if conn_redis.sadd("s_not_200_urls", url):
                    logger.info("Readd url res is\t" + str(conn_redis.lpush(key[:-7], url)) + "\tkey is\t" + key[:-7] + "\t url is \t" + url)
                else:
                    print("Retry already!")
            else:
                logger.info("No url in set \t" + str(key))
            time.sleep(ts)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/5 14:53
# @Email     : billsteve@126.com
# @Des       : 
# @File        : GPNewsItem
# @Software: PyCharm
from scrapy import Field, Item


class GPNewsItem(Item):
    spider = Field()
    title = Field()
    publish_time = Field()
    news_source = Field()
    read_num = Field()
    content = Field()
    url = Field()
    source_url = Field()

    def __init__(self, *arg, **kw):
        super(GPNewsItem, self).__init__(*arg, **kw)
        self["spider"] = None
        self["title"] = None
        self["publish_time"] = None
        self["news_source"] = None
        self["read_num"] = None
        self["content"] = None
        self["source_url"] = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/21 14:07
import pymongo


class MongoHelper():
    def __init__(self, *arg, **kw):
        self.host = kw.get("host", "localhost")
        self.port = kw.get("host", 27017)

    @staticmethod
    def get_connection(host="localhost", port=2707, pwd=None):
        return pymongo.MongoClient(host, port)

    @staticmethod
    def get_connection_db(host="localhost", port=2707, db=None, pwd=None):
        conn = MongoHelper.get_connection(host, port)
        return conn.db

    @staticmethod
    def get_collection(host="localhost", port=2707, db=None, collection_name=None, username=None, password=None):
        db = MongoHelper.get_connection_db(host, port, db)
        return db.collection_name


if __name__ == '__main__':
    import XX.Model.Struct.MongoConn as MC

    config = MC.zhihan00_cfg
    client = MongoHelper.get_connection(**config)
    db = client.dbname
    col = db.col
    print(db)
    print(col)
    print(col.insert_one({"x": 12}).inserted_id)



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/20 23:56
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Engine.py
# @Software : PyCharm
import XX.Multiprocess.Callback as cb


def apply_async_function(pro_num, fun, para=None, error_callback=cb.msg):
    from multiprocessing import Pool
    pool = Pool(pro_num)
    for i in range(pro_num):
        if para:
            if isinstance(para, tuple) or isinstance(para, list):
                ppara = list(para)
                ppara.append(i)
                pool.apply_async(fun, args=ppara, error_callback=error_callback)
            if isinstance(para, dict):
                para["pro_num"] = i
                pool.apply_async(fun, kwds=para, error_callback=error_callback)
        else:
            pool.apply_async(fun, (i,), error_callback=error_callback)
    pool.close()
    pool.join()



# coding: utf-8
from sqlalchemy import Column, Index, Integer, text
from sqlalchemy.dialects.mysql import TEXT, VARCHAR
from sqlalchemy.ext.declarative import declarative_base
from XX.Model.SqlAlchemy.BaseModel import *

Base = declarative_base()
metadata = Base.metadata


class ComChangelog(Base, BaseModel):
    __tablename__ = 'com_changelog'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    web_key = Column(VARCHAR(255))
    ProjectName = Column(TEXT)
    ChangeDate = Column(VARCHAR(255))
    BeforeContent = Column(TEXT)
    AfterContent = Column(TEXT)
    is_del = Column(VARCHAR(255))
    create_ts = Column(VARCHAR(255))
    update_ts = Column(VARCHAR(255))


class ComEmployee(Base, BaseModel):
    __tablename__ = 'com_employees'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    web_key = Column(VARCHAR(255))
    Name = Column(VARCHAR(255))
    Job = Column(VARCHAR(255))
    CerNo = Column(VARCHAR(255))
    ScertName = Column(VARCHAR(255))
    is_del = Column(Integer)
    update_ts = Column(Integer)
    create_ts = Column(Integer)


class ComHolder(Base, BaseModel):
    __tablename__ = 'com_holder'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    invested_id = Column(VARCHAR(60))
    web_key = Column(VARCHAR(255))
    StockName = Column(VARCHAR(255))
    StockType = Column(VARCHAR(255))
    StockPercent = Column(VARCHAR(255))
    IdentifyType = Column(VARCHAR(255))
    IdentifyNo = Column(VARCHAR(255))
    ShouldCapi = Column(VARCHAR(255))
    ShoudDate = Column(VARCHAR(255))
    InvestType = Column(VARCHAR(255))
    InvestName = Column(VARCHAR(255))
    RealCapi = Column(VARCHAR(255))
    CapiDate = Column(VARCHAR(255))
    Org = Column(VARCHAR(255))
    HolderRate = Column(VARCHAR(10))
    is_del = Column(Integer)
    update_ts = Column(Integer)
    create_ts = Column(Integer)


class ComInfo(Base, BaseModel):
    __tablename__ = 'com_info'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer, index=True)
    web_key = Column(VARCHAR(255))
    logo = Column(VARCHAR(255))
    name = Column(VARCHAR(255))
    tel = Column(VARCHAR(255))
    email = Column(VARCHAR(255))
    web_url = Column(VARCHAR(255))
    Address = Column(VARCHAR(255))
    AnnualReports = Column(VARCHAR(255))
    CreditCode = Column(VARCHAR(255))
    taxpayer_no = Column(VARCHAR(255))
    No = Column(VARCHAR(255))
    organization_no = Column(VARCHAR(255), server_default=text("''"))
    OperName = Column(VARCHAR(255))
    RegistCapi = Column(VARCHAR(255))
    Status = Column(VARCHAR(255))
    StartDate = Column(VARCHAR(255))
    EconKind = Column(VARCHAR(255))
    staff_num = Column(VARCHAR(255))
    TermStart = Column(VARCHAR(255))
    TeamEnd = Column(VARCHAR(255))
    BelongOrg = Column(VARCHAR(255))
    CheckDate = Column(VARCHAR(255))
    en_name = Column(VARCHAR(255))
    register_addr = Column(VARCHAR(255))
    industry_belong = Column(VARCHAR(255))
    used_name = Column(VARCHAR(255))
    location = Column(VARCHAR(255))
    Scope = Column(TEXT)
    intro = Column(TEXT)
    EndDate = Column(VARCHAR(255))
    Province = Column(VARCHAR(50))
    Industry = Column(VARCHAR(255))
    ImageUrl = Column(VARCHAR(255))
    OrgNo = Column(VARCHAR(255))
    EnglishName = Column(VARCHAR(255))
    Type = Column(VARCHAR(255))
    Tag = Column(VARCHAR(255))
    Financing = Column(VARCHAR(255))
    id_del = Column(VARCHAR(255))
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.Address = kw.get("Address", None)
        self.AnnualReports = kw.get("AnnualReports", None)
        self.BelongOrg = kw.get("BelongOrg", None)
        self.CheckDate = kw.get("CheckDate", None)
        self.CreditCode = kw.get("CreditCode", None)
        self.EconKind = kw.get("EconKind", None)
        self.EndDate = kw.get("EndDate", None)
        self.EnglishName = kw.get("EnglishName", None)
        self.Financing = kw.get("Financing", None)
        self.ImageUrl = kw.get("ImageUrl", None)
        self.Industry = kw.get("Industry", None)
        self.No = kw.get("No", None)
        self.OperName = kw.get("OperName", None)
        self.OrgNo = kw.get("OrgNo", None)
        self.Province = kw.get("Province", None)
        self.RegistCapi = kw.get("RegistCapi", None)
        self.Scope = kw.get("Scope", None)
        self.StartDate = kw.get("StartDate", None)
        self.Status = kw.get("Status", None)
        self.Tag = kw.get("Tag", None)
        self.TeamEnd = kw.get("TeamEnd", None)
        self.TermStart = kw.get("TermStart", None)
        self.Type = kw.get("Type", None)
        self.com_id = kw.get("com_id", None)
        self.web_key = kw.get("web_key", None)
        self.create_ts = kw.get("create_ts", None)
        self.email = kw.get("email", None)
        self.en_name = kw.get("en_name", None)
        self.get = kw.get("get", None)
        self.getAll = kw.get("getAll", None)
        self.getAllIds = kw.get("getAllIds", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.getByName = kw.get("getByName", None)
        self.id = kw.get("id", None)
        self.id_del = kw.get("id_del", None)
        self.industry_belong = kw.get("industry_belong", None)
        self.intro = kw.get("intro", None)
        self.location = kw.get("location", None)
        self.logo = kw.get("logo", None)
        self.metadata = kw.get("metadata", None)
        self.name = kw.get("name", None)
        self.organization_no = kw.get("organization_no", None)
        self.register_addr = kw.get("register_addr", None)
        self.staff_num = kw.get("staff_num", None)
        self.taxpayer_no = kw.get("taxpayer_no", None)
        self.tel = kw.get("tel", None)
        self.update_ts = kw.get("update_ts", None)
        self.used_name = kw.get("used_name", None)
        self.web_url = kw.get("web_url", None)

    @staticmethod
    def getByComId(com_id, session):
        return session.query(ComInfo).filter(ComInfo.com_id == com_id).limit(1).all()

    @staticmethod
    def getByComKey(web_key, session):
        return session.query(ComInfo).filter(ComInfo.web_key == web_key).limit(1).all()

    @staticmethod
    def saveComInfo(data, session):
        com_id = data["com_id"]
        if com_id:
            exists = ComInfo.getByComId(com_id, session)
            if not exists:
                com_info = ComInfo(**data)
                ComInfo.addModel(com_info, session)
                return com_info.id
            else:
                return exists[0].id


class ComInvest(Base, BaseModel):
    __tablename__ = 'com_invest'
    __table_args__ = (
        Index('wy', 'com_id', 'invest_id', unique=True),
    )

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    invest_id = Column(Integer)
    invest_ts = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)


class ComKey500(Base, BaseModel):
    __tablename__ = 'web_key_500'

    id = Column(Integer, primary_key=True)
    web_key = Column(VARCHAR(65), unique=True)
    has_crawled = Column(Integer)
    firm = Column(VARCHAR(255))
    invest = Column(VARCHAR(255))
    my_id = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    @staticmethod
    def getByComKey(web_key, session):
        return session.query(ComInfo).filter(ComInfo.web_key == web_key).limit(1).all()


class ComKey(Base, BaseModel):
    __tablename__ = 'com_keys'

    id = Column(Integer, primary_key=True)
    com_name = Column(VARCHAR(200))
    short_name = Column(VARCHAR(100))
    web_id = Column(Integer)
    web_url = Column(VARCHAR(100))
    web_key = Column(VARCHAR(100), unique=True)
    cassets = Column(Integer)
    cinvestment = Column(Integer)
    cjob = Column(Integer)
    clawsuit = Column(Integer)
    creport = Column(Integer)
    crun = Column(Integer)
    firm = Column(Integer)
    info = Column(VARCHAR(40))
    is_del = Column(Integer)
    level = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.cassets = kw.get("cassets", None)
        self.cinvestment = kw.get("cinvestment", None)
        self.cjob = kw.get("cjob", None)
        self.clawsuit = kw.get("clawsuit", None)
        self.com_name = kw.get("com_name", None)
        self.create_ts = kw.get("create_ts", None)
        self.creport = kw.get("creport", None)
        self.crun = kw.get("crun", None)
        self.firm = kw.get("firm", None)
        self.get = kw.get("get", None)
        self.getAll = kw.get("getAll", None)
        self.getAllIds = kw.get("getAllIds", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.getByName = kw.get("getByName", None)
        self.id = kw.get("id", None)
        self.info = kw.get("info", None)
        self.is_del = kw.get("is_del", None)
        self.level = kw.get("level", None)
        self.metadata = kw.get("metadata", None)
        self.short_name = kw.get("short_name", None)
        self.update_ts = kw.get("update_ts", None)
        self.web_key = kw.get("web_key", None)
        self.web_url = kw.get("web_url", None)
        self.web_id = kw.get("web_id", None)

    @staticmethod
    def saveComKey(data, session):
        web_key = data["web_key"]
        if web_key:
            exists = ComKey.getByKV("web_key", web_key, session)
            if not exists:
                web_key = ComKey(**data)
                ComKey.addModel(web_key, session)
                return web_key.id
            else:
                return exists[0].id

    @staticmethod
    def saveComKeyAndComInfo(data, session):
        com_id = ComKey.saveComKey(data, session)
        if com_id:
            data["com_id"] = com_id
            add_info = ComInfo.saveComInfo(data, session)
            if add_info:
                return 1


class ComSub(Base, BaseModel):
    __tablename__ = 'com_sub'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    web_key = Column(VARCHAR(255))
    subcom_id = Column(Integer)
    subweb_key = Column(VARCHAR(255))
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)


if __name__ == '__main__':
    createInitFunction(ComSub)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/1/25 16:53
# @File           : UserItemModel
# @Des           : 
# @Email        : billsteve@126.com
import XX.Model.SqlAlchemy.BaseModel as BM
from sqlalchemy import Column, String
from sqlalchemy.dialects.mysql import INTEGER, TINYINT
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class UserItem(Base, BM.BaseModel):
    __tablename__ = 'user_items'

    id = Column(INTEGER(11), primary_key=True)
    uid = Column(INTEGER(11))
    item_name = Column(String(60))
    item_content = Column(String(255))
    is_del = Column(TINYINT(1))
    create_ts = Column(INTEGER(11))
    update_ts = Column(INTEGER(11))

    def __init__(self, *arg, **kw):
        self.create_ts = kw.get("create_ts", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.item_content = kw.get("item_content", None)
        self.item_name = kw.get("item_name", None)
        self.metadata = kw.get("metadata", None)
        self.uid = kw.get("uid", None)
        self.update_ts = kw.get("update_ts", None)


if __name__ == '__main__':
    BM.createInitFunction(UserItem)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/29 15:06
# @Author   : Peter
# @Des       : 
# @File        : CompanyChangeModel
# @Software: PyCharm

from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, Integer, String, Text
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class CompanyChange(Base, BaseModel):
    __tablename__ = 'company_change'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    web_key = Column(String(255, 'utf8mb4_unicode_ci'))
    ProjectName = Column(Text(collation='utf8mb4_unicode_ci'))
    ChangeDate = Column(String(255, 'utf8mb4_unicode_ci'))
    BeforeContent = Column(Text(collation='utf8mb4_unicode_ci'))
    AfterContent = Column(Text(collation='utf8mb4_unicode_ci'))
    is_del = Column(String(255, 'utf8mb4_unicode_ci'))
    create_ts = Column(String(255, 'utf8mb4_unicode_ci'))
    update_ts = Column(String(255, 'utf8mb4_unicode_ci'))

    def __init__(self, *arg, **kw):
        self.AfterContent = kw.get("AfterContent", None)
        self.BeforeContent = kw.get("BeforeContent", None)
        self.ChangeDate = kw.get("ChangeDate", None)
        self.ProjectName = kw.get("ProjectName", None)
        self.com_id = kw.get("com_id", None)
        self.web_key = kw.get("web_key", None)
        self.create_ts = kw.get("create_ts", None)
        self.get = kw.get("get", None)
        self.getAll = kw.get("getAll", None)
        self.getAllIds = kw.get("getAllIds", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.getByName = kw.get("getByName", None)
        self.getColumsByFromIdAndMod = kw.get("getColumsByFromIdAndMod", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.updateId = kw.get("updateId", None)
        self.update_ts = kw.get("update_ts", None)


if __name__ == '__main__':
    createInitFunction(CompanyChange)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/1/17 11:08
# @File           : __init__.py
# @Des           : 
# @Email        : billsteve@126.com

if __name__ == "__main__":
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/2/27 15:35
# @File           : ForumUrlModel
# @Des           : 
# @Email        : billsteve@126.com
from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, Index, String
from sqlalchemy.dialects.mysql import INTEGER, TINYINT
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class ForumUrlModel(Base, BaseModel):
    __tablename__ = 'forum_url'
    __table_args__ = (
        Index('forum_id', 'forum_id', 'url'),
    )

    id = Column(INTEGER(11), primary_key=True)
    forum_id = Column(INTEGER(11))
    forum_name = Column(String(255))
    url = Column(String(255))
    is_del = Column(TINYINT(4))
    create_ts = Column(INTEGER(11))
    update_ts = Column(INTEGER(11))

    def __init__(self, *arg, **kw):
        self.create_ts = kw.get("create_ts", None)
        self.forum_id = kw.get("forum_id", None)
        self.forum_name = kw.get("forum_name", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.update_ts = kw.get("update_ts", None)
        self.url = kw.get("url", None)


if __name__ == '__main__':
    createInitFunction(ForumUrlModel)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/27 10:53
# @Author   : Peter
# @Des       : 
# @File        : RegExp
# @Software: PyCharm
import re

import XX.Date.DatetimeHelper as DT


class RegExpHelper:

    @staticmethod
    def get_url_from_html(txt):
        return re.findall("(?<=href=\").+?(?=\")|(?<=href=\').+?(?=\')", txt)

    @staticmethod
    def get_url_from_text(txt):
        return re.findall("http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", txt)

    @staticmethod
    def get_ips(txt):
        reg_txt = r"\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b"
        return re.findall(reg_txt, txt)
        # return re.findall("((?:(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d)))\.){3}(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d))))", txt)

    @staticmethod
    def replace_continuous_blank_2_one(txt):
        return re.sub(' +', ' ', txt)

    @staticmethod
    def get_number(txt):
        if not txt or not isinstance(txt, str):
            return
        return re.findall(r"(\d+)", txt)

    @staticmethod
    def is_url(txt):
        return re.findall("http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", txt)

    @staticmethod
    def is_domain(txt):
        return re.findall("(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", txt)

    @staticmethod
    def get_date(s):
        if not s or not isinstance(s, str):
            return ""
        p = re.compile(r'\d{4}[-|/|.]\d{1,2}[-|/|.]\d{1,2}|\d{4}年\d{1,2}月\d{1,2}[日号]|今天|\d{1,2}月\d{1,2}[日号]')
        result = re.findall(p, s)
        try:
            print(result[0])
            result[0] = result[0].replace('年', '-').replace('月', '-').replace('日', '').replace('号', '')
            if result[0] == "今天":
                return DT.get_today()
            if len(result[0]) == 4 or len(result[0]) == 5:
                print(result[0])
                return str(DT.get_today().split("-")[0]) + result[0]
            return result[0]
        except Exception as e:
            print(e)

    # TODO:适应  20190202这样的情况
    @staticmethod
    def get_date_time(s):
        if not s or not isinstance(s, str):
            return ""
        r = r"(\d{2,4}[-/年]\d{1,2}[-/月]\d{1,2}日? \d{2}:\d{2}(:\d{2})?)"
        r = r"\d{2,4}[-/年]\d{1,2}[-/月]\d{1,2}日? \d{2}:\d{2}(:\d{2})?"
        p = re.compile(r)
        result = re.findall(p, s)
        try:
            result[0] = result[0].replace('年', '-').replace('月', '-').replace('日', '').replace('号', '')
            if result[0] == "今天":
                return DT.get_today()
            if len(result[0]) == 4 or len(result[0]) == 5:
                print(result[0])
                return str(DT.get_today().split("-")[0]) + result[0]
            return result[0]
        except Exception as e:
            print(e)

    # 是否是汉字
    @staticmethod
    def is_Chinese(w):
        for ch in w:
            if '\u4e00' <= ch <= '\u9fff':
                return True
        return False

    @staticmethod
    def get_chinese(s):
        return re.sub("[A-Za-z0-9\_\{\}\$\.\:\/\"\!\%\[\]\,\。]", "", s)


if __name__ == '__main__':
    r = RegExpHelper()
    s = r.get_ips("192.168.1.1")
    print(s)



# -*- encoding:utf8 -*-
import datetime
import time

import pandas as pd

import XX.funclib as cfun


# 获取距离今天的某段时间的时间戳
def get_add_date_ts(month=0, day=1, start_ts=None):
    if start_ts:
        x = time.localtime(start_ts)
    else:
        x = time.localtime(time.time())
    Y = int(time.strftime('%Y', x))
    M = int(time.strftime('%m', x))
    D = int(time.strftime('%d', x))
    z = datetime.datetime(Y, M, D)
    starts = str(z + pd.tseries.offsets.DateOffset(months=month, days=day))
    ts = int(time.mktime(time.strptime(starts, '%Y-%m-%d %H:%M:%S')))
    return ts


# 字符串转时间戳
def str_to_ts(str1, fmt='%Y-%m-%d %H:%M:%S'):
    return int(time.mktime(time.strptime(str1, fmt)))


# 时间戳转换为日期
def ts_to_date(ts=time.time(), fmt='%Y-%m-%d'):
    return time.strftime(fmt, time.localtime(ts))


# 时间戳转换为日期
def ts_to_datetime(ts=time.time(), fmt='%Y-%m-%d %H:%M:%S'):
    return time.strftime(fmt, time.localtime(ts))


# 时间戳转换为时间
def ts_to_time(ts=time.time(), fmt='%H:%M:%S'):
    return time.strftime(fmt, time.localtime(ts))


# 获取现在的时间
def get_now_time(ts=time.time(), fmt='%Y-%m-%d %H:%M:%S'):
    return time.strftime(fmt, time.localtime(ts))


# 获取今天的日期
def get_today(ts=time.time(), fmt='%Y-%m-%d'):
    return time.strftime(fmt, time.localtime(ts))


# 获取今天的月份
def get_this_month(ts=time.time(), fmt='%Y-%m'):
    return time.strftime(fmt, time.localtime(ts))


# 获取当前小时
def get_hours(ts=time.time()):
    return time.strftime('%H', time.localtime(ts))


# 获取计算时间
def get_calc_date(s):
    return cfun.str_to_standard_time(s)


def is_date(date):
    try:
        date = date.strip()
        if len(date) == 0:
            return False
        if ":" in date:
            if date.count(":") == 1:
                time.strptime(date, "%Y-%m-%d %H:%M")
            elif date.count(":") == 2:
                time.strptime(date, "%Y-%m-%d %H:%M:%S")
        elif "-" in date:
            time.strptime(date, "%Y-%m-%d")
        elif "/" in date:
            time.strptime(date, "%Y/%m/%d")
        elif "." in date:
            time.strptime(date, "%Y.%m.%d")
        elif len(date) == 8:
            time.strptime(date, "%Y%m%d")
        return True
    except Exception as e:
        return False


def get_date(date):
    date = date.replace("年", "-").replace("月", "-").replace("日", "").replace("号", "").replace("点", ":").replace("时",
                                                                                                                ":").replace(
        "分", ":").replace("秒", "")
    date = cfun.str_to_standard_time(date)
    try:
        date = date.strip()
        if len(date) == 0:
            return False
        date = cfun.str_to_standard_time(date)
        if ":" in date:
            if date.count(":") == 1:
                date = time.strptime(date, "%Y-%m-%d %H:%M")
            elif date.count(":") == 2:
                date = time.strptime(date, "%Y-%m-%d %H:%M:%S")
        elif "-" in date:
            if len(date.split("-")[0]) == 4:
                # "2018-01-01"
                date = time.strptime(date, "%Y-%m-%d")
            elif len(date.split("-")[0]) == 2 and date.count("-") == 2:
                date = time.strptime("20" + date, "%Y-%m-%d")
            elif len(date.split("-")[0]) == 2 and date.count("-") == 1:
                date = time.strptime(get_today()[:4] + "-" + date, "%Y-%m-%d")
        elif "/" in date:
            date = time.strptime(date, "%Y/%m/%d")
        elif "." in date:
            date = time.strptime(date, "%Y.%m.%d")
        elif len(date) == 8:
            date = time.strptime(date, "%Y%m%d")
        else:
            date = ""
        return time.strftime('%Y-%m-%d', date)
    except Exception as e:
        print(e)
        return ""


# 时间戳转为iso8601时间
def ts_to_iso8601(ts=time.time(), format_='%Y-%m-%dT%H:%M:%S.%fZ'):
    format_ = format_.replace('%f', '{-FF-}')  # 订单处理微秒数据 %f
    length = min(16, len(str(ts)))  # 最多去到微秒级

    # 获取毫秒/微秒 数据
    sec = '0'
    if length != 10:  # 非秒级
        sec = str(ts)[:16][-(length - 10):]  # 最长截取16位长度 再取最后毫秒/微秒数据
    sec = '{:0<6}'.format(sec)  # 长度位6，靠左剩下的用0补齐
    timestamp = float(str(ts)[:10])  # 转换为秒级时间戳
    return datetime.datetime.utcfromtimestamp(timestamp).strftime(format_).replace('{-FF-}', sec)


if __name__ == '__main__':
    print(ts_to_iso8601())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2019/1/3 10:25
# @Email     : billsteve@126.com
# @Des       : 
# @File        : WeixinModel
# @Software: PyCharm
from sqlalchemy import Column, Date, String
from sqlalchemy.dialects.mysql import INTEGER, TINYINT
from sqlalchemy.ext.declarative import declarative_base
import XX.Model.SqlAlchemy.BaseModel as BaseModel

Base = declarative_base()
metadata = Base.metadata


class WeixinModel(Base, BaseModel.BaseModel):
    __tablename__ = 'weixin'

    id = Column(INTEGER(11), primary_key=True)
    wx_id = Column(INTEGER(11))
    biz = Column(String(32), unique=True)
    name = Column(String(255))
    gh_id = Column(INTEGER(11))
    weixin_id = Column(INTEGER(11))
    head_img = Column(String(255))
    head_img_circle = Column(String(255))
    intro = Column(String(255))
    no1 = Column(INTEGER(11))
    no2 = Column(String(255))
    no3 = Column(INTEGER(11))
    no4 = Column(INTEGER(11))
    no5 = Column(INTEGER(11))
    is_del = Column(TINYINT(1))
    update_Ts = Column(INTEGER(11))
    create_ts = Column(INTEGER(11))

    def __init__(self, *arg, **kw):
        self.biz = kw.get("biz", None)
        self.create_ts = kw.get("create_ts", None)
        self.gh_id = kw.get("gh_id", None)
        self.head_img = kw.get("head_img", None)
        self.head_img_circle = kw.get("head_img_circle", None)
        self.id = kw.get("id", None)
        self.intro = kw.get("intro", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.name = kw.get("name", None)
        self.no1 = kw.get("no1", None)
        self.no2 = kw.get("no2", None)
        self.no3 = kw.get("no3", None)
        self.no4 = kw.get("no4", None)
        self.no5 = kw.get("no5", None)
        self.update_Ts = kw.get("update_Ts", None)
        self.weixin_id = kw.get("weixin_id", None)
        self.wx_id = kw.get("wx_id", None)


if __name__ == '__main__':
    BaseModel.createInitFunction(WeixinModel)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 放到MySQL数据库


class MysqlPipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        cls.cacheFilePath = cls.settings.get("FUN_CACHE_FILE_PATH")
        return cls()

    def process_item(self, item, spider):
        import importlib
        module = importlib.import_module(self.settings.get('JSON2MSYQL_MODEL_PATH'), self.settings.get("PROJECT_PATH"))
        getattr(module, spider.name)(item, self.settings.get("mysql_cfg"))
        return item



# -*- coding: utf-8 -*-
import time
import traceback

import redis
from logzero import logger

import XX.DB.MysqlHelper as MH
import XX.DB.RedisHelper as  RH
import XX.File.FileHelper as FH


def get_key_type(k):
    if k.startswith("l"):
        t = "list"
    elif k.startswith("s"):
        t = "set"
    elif k.startswith("k"):
        t = "kv"
    elif k.startswith("z"):
        t = "zset"
    elif k.startswith("h"):
        t = "hash"
    else:
        t = None
    return t


def get_key_value(key, conn):
    key_type = get_key_type(key)
    if key_type == "list":
        return conn.lpop(key)
    elif key_type == "set":
        return conn.spop(key)
    elif key_type == "kv":
        return conn.get(key)
    elif key_type == "zset":
        return conn.zrange(key, 0, 1)
    elif key_type == "hash":
        return conn.hget(key)
    else:
        return None


def set_key_value(key, val, r, *arg, **kw):
    key_type = get_key_type(key)
    if key_type == "list":
        return r.rpush(key, val)
    elif key_type == "set":
        return r.sadd(key, val)
    elif key_type == "kv":
        return r.set(key, val)
    elif key_type == "zset":
        if kw.get("score"):
            return r.zset(key, val, kw.get("score"))
        return
    elif key_type == "hash":
        return r.hset(key, val)
    else:
        print(" Not allowed keyType " + key)
        return None


# DB 2 Redis
# TODO:优化测试 2021年1月19日11:50:42
class Table2Redis():
    def __init__(self, *argvs, **kw):
        self.db = kw.get("mysql_cfg", {}).get("db")
        self.ConnRedis = RH.RedisHelper.get_redis_connect_by_cfg(kw.get("redis_cfg"))
        self.ConnMysql = MH.MysqlHelper(**kw.get("mysql_cfg"))

    # 把某一列加到redis中:
    # TODO: add zset hash other.......设计其他的类型
    def column_to_redis(
            self, table_name, column, key_name="",
            key_type="list",
            exists_set_name=None,
            from_0=False,
            del_key=False,
            del_exists_key=False,
            total=0, ts=0,
            key_filter=None,
            key_name_filter=None, *arg, **kw):
        from_id_key = "k_add_" + self.db + "_" + table_name + "_" + key_name + "_from_id"
        redis_from_id = self.ConnRedis.get(from_id_key) if self.ConnRedis.get(from_id_key) else 0
        from_id = 0 if from_0 else redis_from_id
        k_ADDREDIS_FROM_ID = from_id if from_id else 0
        print(" ==== FROM_ID is " + str(from_id))

        sql = "Select id," + column + " FROM " + table_name + " WHERE  ID > %s LIMIT 1000"
        keys = 1
        if del_key:
            self.ConnRedis.delete(key_name)
        if del_exists_key:
            self.ConnRedis.delete(exists_set_name)
        num = 0
        key_type = key_type if key_type else get_key_type(key_name)
        while keys:
            val = k_ADDREDIS_FROM_ID,
            # TODO:
            keys = self.ConnMysql.get_lists(sql, val)
            for one_key in keys:
                key = list(one_key)
                if key_filter:
                    key[1] = key_filter(key[1], **kw)
                k_ADDREDIS_FROM_ID = key[0]
                if key[1]:
                    if key_name_filter:
                        KeyName = key_name_filter(KeyName=KeyName, key=key[1])
                    if ts and ts < 0.01:
                        if k_ADDREDIS_FROM_ID % int(1 / ts) == 0:
                            time.sleep(1)
                    else:
                        time.sleep(ts)
                    res = 0
                    if exists_set_name:
                        if self.ConnRedis.sismember(exists_set_name, key[1]):
                            print("EXITS >>> " + key[1] + "  from _id >>  " + str(key[0]))
                            continue
                    if key_type == "list":
                        if kw.get("limit"):
                            if int(self.ConnRedis.llen(key)) > kw["limit"]:
                                print("Too Much")
                                return
                        # 把某列添加到队列中
                        if len(key) == 2:
                            res = self.ConnRedis.rpush(KeyName, key[1])
                        # 某两列中的一列添加到队列中
                        elif len(key) == 3:
                            value = key[1] if key[1] else key[2]
                            res = self.ConnRedis.lpush(KeyName, value)
                        # 某三列中的一列添加到队列中
                        elif len(key) == 4:
                            value = key[1] if key[1] else key[2] if key[2] else key[3]
                            res = self.ConnRedis.lpush(KeyName, value)
                    elif key_type == "set":
                        if kw.get("limit"):
                            if int(self.ConnRedis.scard(key)) > kw["limit"]:
                                print("Too Much")
                                return
                        res = self.ConnRedis.sadd(KeyName, key[1])
                    elif key_type == "kv":
                        res = self.ConnRedis.set(key[1], key[2])
                    else:
                        logger.error(f"key_type is not support : {key_type}")
                    if res and exists_set_name:
                        self.ConnRedis.sadd(exists_set_name, key[1])
                    self.ConnRedis.set(from_id_key, k_ADDREDIS_FROM_ID)
                    print(kw.get("fn") + " add res :>> " + str(res) + "  now id :>> " + str(key[0]))
                    num += 1
                    if total and num >= total:
                        print(" Done --- ")
                        return
                else:
                    print("Error key" + str(key[1]))


# Redis 2 Redis
class Redis2Redis():
    def __init__(self, Host1="localhost", Host2="localhost", db1=0, db2=0, *arg, **kw):
        if not db2:
            db2 = db1
        self.r1 = redis.Redis(Host1, db=db1, decode_responses=True)
        self.r2 = redis.Redis(Host2, db=db2, decode_responses=True)

    def r1_2_r2(self, key1, key2, exists_set_name=None, total=0, key_filter=None, delete_source=True, ts=0, *arg, **kw):
        Total = total if not delete_source and total else total if delete_source else 1000
        val = 1
        num = 0
        while val:
            val = get_key_value(key1, self.r1)
            if key_filter:
                val = key_filter(val, *arg, **kw)
            if val:
                if exists_set_name:
                    if not self.r2.sadd(exists_set_name, val):
                        continue
                if set_key_value(key2, val, self.r2):
                    if not delete_source:
                        set_key_value(key1, val, self.r1)
                    print("ok")
                    num += 1
                    if Total and num >= Total:
                        break
                else:
                    print(" fail to add val.")


class Redis2File():

    def __init__(self, Host="localhost", db=0, *arg, **kw):
        self.r = redis.Redis(Host, db=db, decode_responses=True)

    def r_2_file(self, key, FilePath, ExistsSetName=None, Total=0, KeyFilter=None, DeleteSource=True, ts=0, LineEnd="",
                 *arg, **kw):
        Total = Total if not DeleteSource and Total else Total if DeleteSource else 1000
        val = 1
        num = 0
        while val:
            val = get_key_value(key, self.r)
            val = KeyFilter(val, *arg, **kw) if KeyFilter else val
            try:
                if val:
                    if ExistsSetName:
                        if not self.r.sadd(ExistsSetName, val):
                            continue
                    if FH.FileHelper.save_file(FilePath, str(val)):
                        if not DeleteSource:
                            set_key_value(key, val, self.r)
                        print("ok  =  " + str(num))
                        num += 1
                        if Total and num >= Total:
                            break
                    else:
                        print(" fail to add val.")
                else:
                    print(" over ")
            except:
                set_key_value(key, val, self.r)
                traceback.print_exc()


if __name__ == "__main__":
    local = "localhost"
    pass
    # 把info 和 info12315的添加进去
    # redisHelper = AddRedisHelper(MYSQL_HOST13, MYSQL_USER13, MYSQL_PWD13, "gov", REDIS_HOST13, 9)
    # redisHelper.AddColumnToRedis("info12315", "REGNO", KeyName="l_123415_to_crawl_code", ExistsSetName="s_123415_crawled_code")
    # redisHelper.AddColumnToRedis("info", "REGNO", "l_123415_to_crawl_code", "s_123415_crawled_code")

    # 把qcc的号添加进去
    # redisHelper = AddRedisHelper(MYSQL_HOST13, MYSQL_USER13, MYSQL_PWD13, "qcc", REDIS_HOST13, 9)
    # redisHelper.AddColumnToRedis("com_info", "No,CreditCode", KeyName="l_123415_to_crawl_code", ExistsSetName="s_123415_crawled_code")

    # 吧info123添加到 已抓列表
    # redisHelper = AddRedisHelper(MYSQL_HOST13, MYSQL_USER13, MYSQL_PWD13, "gov", REDIS_HOST13, 9)
    # redisHelper.AddColumnToRedis("info", "REGNO", "s_123415_crawled_code", KeyType="set", From0=True)

    # qcc 添加
    # redisHelper = AddRedisHelper(
    #     MYSQL_HOST13, MYSQL_USER13, MYSQL_PWD13, "qcc", REDIS_HOST13, 9)
    # redisHelper.AddColumnToRedis(
    #     "com_info", "No,CreditCode", KeyName="s_123415_crawled_code", KeyType="set", From0=True)

    # r2r = Redis2RedisHelper(Host1=local, Host2=local, db1=1, db2=2)
    # r2r.R12R2("l_test", "l_test",ts=0.1,DeleteSource=False)

    # def addN(key):
    #     return str(key)+"\n" if key else key

    # r2f = Redis2FileHelper(local, 2)
    # r2f.R2File("l_test", "E:\\l_test.log", Total=100, KeyFilter=addN)
    import XX.Model.Struct.MysqlConn as MC
    import XX.Model.Struct.RedisConn as RC

    t2r = Table2Redis(mysql_cfg=MC.qq_cfg(db="weibo"), redis_cfg=RC.ali_cfg(db=4))
    t2r.column_to_redis("weibo_info", "web_id", "s_crawled_wb_web_id", "set", limit=10, kv=False)
